@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, unicode_literals, division
 import six
+from collections import OrderedDict
 
 
-class Registry(dict):
+class Registry(OrderedDict):
     """A rule registry dictionary.
 
     The key format is (<bearer>[, <target>][, <permission>])
@@ -14,7 +15,14 @@ class Registry(dict):
         # inheritance to find a rule.
         strs = six.string_types
         cls = key[-2] if isinstance(key[-1], strs) else key[-1]
-        for bkey, base_rule in six.iteritems(self):
+        for bkey in reversed(list(self.keys())):
+            base_rule = self[bkey]
+            if isinstance(key[-1], strs) and key[-1] != bkey[-1]:
+                continue
+
+            if not isinstance(key[-1], strs) and isinstance(bkey[-1], strs):
+                continue
+
             bcls = bkey[-2] if isinstance(bkey[-1], strs) else bkey[-1]
             if issubclass(cls, bcls):
                 break
